//
//  RootViewController.m
//  Hacker News
//
//  Created by Mark Fayngersh on 4/12/10.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "RootViewController.h"
#import "DetailViewController.h"

#import "JSON.h";

/*
 This template does not ensure user interface consistency during editing operations in the table view. You must implement appropriate methods to provide the user experience you require.
 */

@interface RootViewController ()
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath;
@end

@implementation RootViewController

@synthesize detailViewController, fetchedResultsController, managedObjectContext;

#pragma mark -
#pragma mark View lifecycle

- (NSArray *) fetchCurrentObjects {
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Article" inManagedObjectContext:managedObjectContext];
    [fetchRequest setEntity:entity];
	
    NSError *error;
    NSArray *items = [managedObjectContext executeFetchRequest:fetchRequest error:&error];
	[fetchRequest release];
	
	return items;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    editButton = [[UIBarButtonItem alloc] initWithTitle:@"Edit" style:UIBarButtonSystemItemEdit target:self action:@selector(editMode:)];
    self.navigationItem.rightBarButtonItem = editButton;
    self.clearsSelectionOnViewWillAppear = NO;
    self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
	
	rowsToDelete = [[NSMutableArray alloc] init];

    NSError *error = nil;
    if (![[self fetchedResultsController] performFetch:&error]) {
        HNLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }
}

- (void)editMode:(id)sender {
    if (self.tableView.editing) {
        [self.tableView setEditing:NO animated:YES];
        [sender setTitle:@"Edit"];
		
		for(NSManagedObject *object in rowsToDelete) {
			if (detailViewController.detailItem == object) {
				detailViewController.detailItem = nil;
			}
			
			NSManagedObjectContext *context = [fetchedResultsController managedObjectContext];
			[context deleteObject:object];
			
			NSError *error;
			if (![context save:&error]) {
				HNLog(@"Unresolved error %@, %@", error, [error userInfo]);
				abort();
			}
		}
		if([rowsToDelete count] > 0) {
			[self.tableView reloadData];
			[rowsToDelete removeAllObjects];
		}
    } else {
        [self.tableView setEditing:YES animated:YES];
        [sender setTitle:@"Delete"];
	}
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    NSManagedObject *managedObject = [fetchedResultsController objectAtIndexPath:indexPath];
    cell.textLabel.text = [[managedObject valueForKey:@"title"] description];
	cell.textLabel.textColor = ([[managedObject valueForKey:@"favorite"] boolValue]) ? [UIColor redColor] : [UIColor blackColor];
}

#pragma mark -
#pragma mark Add a new object

- (void)insertNewObject:(NSDictionary *)articles favorite:(NSNumber *)isFavorite {	
    NSArray *currentObjects = [self fetchCurrentObjects];
    	
	for (NSDictionary *article in articles) {
        int flag = 0;
        for (NSManagedObject *object in currentObjects) {
            if ([[article valueForKey:@"title"] isEqualToString:[object valueForKey:@"title"]]) {
                flag = 1;
            }
        }
		if (flag == 0) {
            NSIndexPath *currentSelection = [self.tableView indexPathForSelectedRow];
            if (currentSelection != nil) {
                [self.tableView deselectRowAtIndexPath:currentSelection animated:NO];
            }    
			
            // Create a new instance of the entity managed by the fetched results controller.
            NSManagedObjectContext *context = [fetchedResultsController managedObjectContext];
            NSEntityDescription *entity = [[fetchedResultsController fetchRequest] entity];
            NSManagedObject *newManagedObject = [NSEntityDescription insertNewObjectForEntityForName:[entity name] inManagedObjectContext:context];
            NSError *error = nil;

            // If appropriate, configure the new managed object.
            [newManagedObject setValue:[article valueForKey:@"title"] forKey:@"title"];
            [newManagedObject setValue:[article valueForKey:@"link"] forKey:@"url"];
            [newManagedObject setValue:[article valueForKey:@"comments_link"] forKey:@"discuss"];
            [newManagedObject setValue:[article valueForKey:@"author"] forKey:@"author"];
            [newManagedObject setValue:[article valueForKey:@"comments_number"] forKey:@"comments"];
            [newManagedObject setValue:isFavorite forKey:@"favorite"];
            
            // Save the context.
            if (![context save:&error]) {
				/*
				 Replace this implementation with code to handle the error appropriately.
				 
				 abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
				 */
                HNLog(@"Unresolved error %@, %@", error, [error userInfo]);
                abort();
            }
			
            NSIndexPath *insertionPath = [fetchedResultsController indexPathForObject:newManagedObject];
            [self.tableView selectRowAtIndexPath:insertionPath animated:YES scrollPosition:UITableViewScrollPositionTop];
            detailViewController.detailItem = newManagedObject;
        }
    }  
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [fetchedResultsController sections].count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [[[fetchedResultsController sections] objectAtIndex:section] numberOfObjects];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }

    // Configure the cell.
    [self configureCell:cell atIndexPath:indexPath];
    
    return cell;
}

-(UITableViewCellEditingStyle)tableView:(UITableView*)tableView editingStyleForRowAtIndexPath:(NSIndexPath*)indexPath {
	return 3;
}

#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)aTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Set the detail item in the detail view controller.
    NSManagedObject *selectedObject = [[self fetchedResultsController] objectAtIndexPath:indexPath];
    detailViewController.detailItem = selectedObject;    

	if (self.tableView.editing) {
		[rowsToDelete addObject:selectedObject];
	}
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath {
	NSManagedObject *deselectedObject = [[self fetchedResultsController] objectAtIndexPath:indexPath];
	if (self.tableView.editing) {
		[rowsToDelete removeObject:deselectedObject];
	}
}

#pragma mark -
#pragma mark Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController {
    if (fetchedResultsController != nil) {
        return fetchedResultsController;
    }
    
    /*
     Set up the fetched results controller.
     */
    // Create the fetch request for the entity.
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Article" inManagedObjectContext:managedObjectContext];
    [fetchRequest setEntity:entity];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Edit the sort key as appropriate.
	 NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"title" ascending:YES];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:managedObjectContext sectionNameKeyPath:nil cacheName:@"Root"];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
    [aFetchedResultsController release];
    [fetchRequest release];
    [sortDescriptor release];
    [sortDescriptors release];
    
    return fetchedResultsController;
}

#pragma mark -
#pragma mark Fetched results controller delegate

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type {
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type newIndexPath:(NSIndexPath *)newIndexPath {
    UITableView *tableView = self.tableView;
    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView endUpdates];
}

#pragma mark -
#pragma mark Memory management

- (void)dealloc {
    [detailViewController release];
    [fetchedResultsController release];
    [managedObjectContext release];
    [editButton release];
    
    [super dealloc];
}

@end
